@@ -3,9 +3,9 @@
 #include <cstring>
 
 #include <algorithm>
+#include <new>
 #include <numeric>
 #include <tuple>
-#include <new>
 
 #include "cpu_cell_list.hpp"
 
@@ -20,7 +20,7 @@ void vesin::cpu::neighbors(
 ) {
     auto cell_list = CellList(cell, options.cutoff);
 
-    for (size_t i=0; i<n_points; i++) {
+    for (size_t i = 0; i < n_points; i++) {
         cell_list.add_point(i, points[i]);
     }
 
@@ -49,8 +49,7 @@ void vesin::cpu::neighbors(
                     return;
                 }
 
-                if ((shift[0] + shift[1] + shift[2] == 0)
-                    && (shift[2] < 0 || (shift[2] == 0 && shift[1] < 0))) {
+                if ((shift[0] + shift[1] + shift[2] == 0) && (shift[2] < 0 || (shift[2] == 0 && shift[1] < 0))) {
                     // drop shifts in the negative half plane or the negative
                     // shift[1] axis. See below for a graphical representation:
                     // we are keeping the shifts indicated with `O` and dropping
@@ -102,7 +101,6 @@ void vesin::cpu::neighbors(
 /// cells with a small bounding box and a large cutoff
 #define MAX_NUMBER_OF_CELLS 1e5
 
-
 /// Function to compute both quotient and remainder of the division of a by b.
 /// This function follows Python convention, making sure the remainder have the
 /// same sign as `b`.
@@ -133,8 +131,7 @@ divmod(std::array<int32_t, 3> a, std::array<size_t, 3> b) {
 CellList::CellList(BoundingBox box, double cutoff):
     n_search_({0, 0, 0}),
     cells_shape_({0, 0, 0}),
-    box_(box)
-{
+    box_(box) {
     auto distances_between_faces = box_.distances_between_faces();
 
     auto n_cells = Vector{
@@ -173,14 +170,14 @@ CellList::CellList(BoundingBox box, double cutoff):
         static_cast<size_t>(n_cells[2]),
     };
 
-    for (size_t spatial=0; spatial<3; spatial++) {
+    for (size_t spatial = 0; spatial < 3; spatial++) {
         if (n_search_[spatial] < 1) {
             n_search_[spatial] = 1;
         }
 
         // don't look for neighboring cells if we have only one cell and no
         // periodic boundary condition
-        if (n_cells[spatial] == 1 && !box.periodic()) {
+        if (n_cells[spatial] == 1 && !box.periodic(spatial)) {
             n_search_[spatial] = 0;
         }
     }
@@ -198,27 +195,23 @@ void CellList::add_point(size_t index, Vector position) {
         static_cast<int32_t>(std::floor(fractional[2] * static_cast<double>(cells_shape_[2]))),
     };
 
-    // deal with pbc by wrapping the atom inside if it was outside of the
-    // cell
+    // deal with pbc by wrapping the atom inside if it was outside of the cell
     CellShift shift;
-    // auto (shift, cell_index) =
-    if (box_.periodic()) {
-        auto result = divmod(cell_index, cells_shape_);
-        shift = CellShift{std::get<0>(result)};
-        cell_index = std::get<1>(result);
-    } else {
-        shift = CellShift({0, 0, 0});
-        cell_index = std::array<int32_t, 3>{
-            std::clamp(cell_index[0], 0, static_cast<int32_t>(cells_shape_[0] - 1)),
-            std::clamp(cell_index[1], 0, static_cast<int32_t>(cells_shape_[1] - 1)),
-            std::clamp(cell_index[2], 0, static_cast<int32_t>(cells_shape_[2] - 1)),
-        };
+    for (size_t spatial = 0; spatial < 3; spatial++) {
+        if (box_.periodic(spatial)) {
+            auto result = divmod(cell_index[spatial], cells_shape_[spatial]);
+            shift[spatial] = std::get<0>(result);
+            cell_index[spatial] = std::get<1>(result);
+        } else {
+            shift[spatial] = 0;
+            cell_index[spatial] = std::clamp(cell_index[spatial], 0, static_cast<int32_t>(cells_shape_[spatial] - 1));
+        }
     }
 
     this->get_cell(cell_index).emplace_back(Point{index, shift});
 }
 
-
+// clang-format off
 template <typename Function>
 void CellList::foreach_pair(Function callback) {
     for (int32_t cell_i_x=0; cell_i_x<static_cast<int32_t>(cells_shape_[0]); cell_i_x++) {
@@ -244,7 +237,10 @@ void CellList::foreach_pair(Function callback) {
                     auto shift = CellShift{cell_shift} + atom_i.shift - atom_j.shift;
                     auto shift_is_zero = shift[0] == 0 && shift[1] == 0 && shift[2] == 0;
 
-                    if (!box_.periodic() && !shift_is_zero) {
+                    if ((shift[0] != 0 && !box_.periodic(0)) ||
+                        (shift[1] != 0 && !box_.periodic(1)) ||
+                        (shift[2] != 0 && !box_.periodic(2)))
+                    {
                         // do not create pairs crossing the periodic
                         // boundaries in a non-periodic box
                         continue;
@@ -269,10 +265,10 @@ CellList::Cell& CellList::get_cell(std::array<int32_t, 3> index) {
                         + index[0];
     return cells_[linear_index];
 }
+// clang-format on
 
 /* ========================================================================== */
 
-
 void GrowableNeighborList::set_pair(size_t index, size_t first, size_t second) {
     if (index >= this->capacity) {
         this->grow();
@@ -312,7 +308,7 @@ void GrowableNeighborList::set_vector(size_t index, vesin::Vector vector) {
 
 template <typename scalar_t, size_t N>
 static scalar_t (*alloc(scalar_t (*ptr)[N], size_t size, size_t new_size))[N] {
-    auto* new_ptr = reinterpret_cast<scalar_t (*)[N]>(std::realloc(ptr, new_size * sizeof(scalar_t[N])));
+    auto* new_ptr = reinterpret_cast<scalar_t(*)[N]>(std::realloc(ptr, new_size * sizeof(scalar_t[N])));
 
     if (new_ptr == nullptr) {
         throw std::bad_alloc();
@@ -351,7 +347,7 @@ void GrowableNeighborList::grow() {
         new_shifts = alloc<int32_t, 3>(neighbors.shifts, neighbors.length, new_size);
     }
 
-    double *new_distances = nullptr;
+    double* new_distances = nullptr;
     if (options.return_distances) {
         new_distances = alloc<double>(neighbors.distances, neighbors.length, new_size);
     }
@@ -429,7 +425,8 @@ void GrowableNeighborList::sort() {
     std::iota(std::begin(indices), std::end(indices), 0);
 
     struct compare_pairs {
-        compare_pairs(size_t (*pairs_)[2]): pairs(pairs_) {}
+        compare_pairs(size_t (*pairs_)[2]):
+            pairs(pairs_) {}
 
         bool operator()(int64_t a, int64_t b) const {
             if (pairs[a][0] == pairs[b][0]) {
@@ -521,9 +518,8 @@ void GrowableNeighborList::sort() {
     }
 }
 
-
 void vesin::cpu::free_neighbors(VesinNeighborList& neighbors) {
-    assert(neighbors.device == VesinCPU);
+    assert(neighbors.device.type == VesinCPU);
 
     std::free(neighbors.pairs);
     std::free(neighbors.shifts);
